Package: GCCfactor
Type: Package
Title: GCC Estimation of the Multilevel Factor Model
Version: 1.2.1
Authors@R: c(
    person("Rui", "Lin", email = "ruilin1081@gmail.com", role = c("aut", "cre")),
    person("Yongcheol", "Shin", email = "yongcheol.shin@york.ac.uk", role = "aut")
    )
Maintainer: Rui Lin <ruilin1081@gmail.com>
Description: Provides methods for model selection, estimation, inference, and simulation for the 
    multilevel factor model, based on the principal component estimation and generalised 
    canonical correlation approach. Details can be found in "Generalised Canonical Correlation 
    Estimation of the Multilevel Factor Model." Lin and Shin (2025) <doi:10.2139/ssrn.4295429>.
Imports: stats, stringr, sandwich, Matrix, reshape2
Suggests: plm
License: GPL (>= 3)
Encoding: UTF-8
LazyData: true
Depends: R (>= 2.10)
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-27 13:23:01 UTC; rl1081
Author: Rui Lin [aut, cre],
  Yongcheol Shin [aut]
Repository: CRAN
Date/Publication: 2026-07-28 11:30:02 UTC
